local L = AceLibrary("AceLocale-2.2"):new("MageAnnounce")
local waterfall = AceLibrary("Waterfall-1.0")
local db

MageAnnounce = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")

------------ Main Code ------------
local isInParty, isInRaid, inBG, inInstance
local WarnNum = 1

function MageAnnounce:OnInitialize()
	------------ Slash Commands ------------
	self.options = { 
		type = "group",
		desc = L["Options"],
		args = {
			menu = {
				type = "execute",
				name = L["Options Menu"],
				desc = L["Shows the Options Menu"],
				wfHidden = true,
				func = function()
						waterfall:Open("MageAnnounce")
					end
			},
			bgenable = {
				type = "toggle",
				name = L["Enable in BGs"],
				desc = L["Enable messages in Battlegrounds"],
				get = function() return db.showInBGs end,
				set = function ()db.showInBGs = not db.showInBGs end,
			},
			showoutside = {
				type = "toggle",
				name = L["Enable Outside Instances"],
				desc = L["Enable messages in Outside 5man and raid instances"],
				get = function() return db.showOutside end,
				set = function ()db.showOutside = not db.showOutside end,
			},
			customchan = {
				type = "text",
				name = L["Custom Channel"],
				desc = L["Custom channel to show messages in"],
				usage = L["<Channel>"],
				get = function() return db.customChan end,
				set = "SetCustomChan",
			},
			spellops = {
				type = "group",
				name = L["Spell Options"],
				order = 1,
				desc = L["Show Spell Options"],
				args = {
					polymsg = { -- Options for Polymorph
						type = "text",
						name = L["Poly Message"],
						desc = L["Message to be displayed when polymorphing"],
						usage = L["<Your message>"],
						order = 1,
						get = function() return db.polyMessage end,
						set = function (newValue)db.polyMessage = newValue end,
					},
					focusmacro = { 
						type = "toggle",
						name = L["I use a focus macro"],
						desc = L["Check this if you use a focus macro for sheeping"],
						order = 2,
						get = function() return db.focusMacro end,
						set = function ()db.focusMacro = not db.focusMacro end,
					},
					polyparty = { 
						type = "toggle",
						name = L["Show Polymorph Messages in Party"],
						desc = L["Toggles the display of polymorph messages in party chat"],
						order = 3,
						get = function() return db.showPolyInParty end,
						set = function ()db.showPolyInParty = not db.showPolyInParty end,
					},
					polyraid = {
						type = "toggle",
						name = L["Show Polymorph Messages in Raid"],
						desc = L["Toggles the display of polymorph messages in raid chat"],
						order = 4,
						get = function() return db.showPolyInRaid end,
						set = function ()db.showPolyInRaid = not db.showPolyInRaid end,
					},
					polycust = {
						type = "toggle",
						name = L["Show Polymorph Messages in Custom Channel"],
						desc = L["Toggles the display of polymorph messages in the custom channel"],
						order = 5,
						get = function() return db.showPolyInCust end,
						set = function ()db.showPolyInCust = not db.showPolyInCust end,
					},
					polywarn = { 
						type = "text",
						name = L["Poly warning message"],
						desc = L["Message to be sent to party members who target your target when polymorphing via wisper"],
						usage = L["<Your message>"],
						order = 6,
						get = function() return db.polyWarning end,
						set = function (newValue)db.polyWarning = newValue end,
					},
					polywhisper = { 
						type = "toggle",
						name = L["Send Polymorph Warning Messages via whisper"],
						desc = L["Toggles the send of polymorph warning via whisper"],
						order = 7,
						get = function() return db.sendPolyWhisper end,
						set = function ()db.sendPolyWhisper = not db.sendPolyWhisper end,
					},
					csmsg = { -- Options for Counter Spell
						type = "text",
						name = L["Counterspell Message"],
						desc = L["Message to be displayed when counterspelling"],
						usage = L["<Your message>"],
						order = 8,
						get = function() return db.csMessage end,
						set = function (newValue)db.csMessage = newValue end,
					},
					csparty = { 
						type = "toggle",
						name = L["Show Counterspell Messages in Party"],
						desc = L["Toggles the display of counterspell messages in party chat"],
						order = 9,
						get = function() return db.showCSInParty end,
						set = function ()db.showCSInParty = not db.showCSInParty end,
					},
					csraid = {
						type = "toggle",
						name = L["Show Counterspell Messages in Raid"],
						desc = L["Toggles the display of counterspell messages in raid chat"],
						order = 10,
						get = function() return db.showCSInRaid end,
						set = function ()db.showCSInRaid = not db.showCSInRaid end,
					},
					cscust = {
						type = "toggle",
						name = L["Show Counterspell Messages in Custom Channel"],
						desc = L["Toggles the display of counterspell messages in the custom channel"],
						order = 11,
						get = function() return db.showCSInCust end,
						set = function ()db.showCSInCust = not db.showCSInCust end,
					},
					slowmsg = { -- Options for Slow
						type = "text",
						name = L["Slow Message"],
						desc = L["Message to be displayed when slowing"],
						usage = L["<Your message>"],
						order = 12,
						get = function() return db.slowMessage end,
						set = function (newValue)db.slowMessage = newValue end,
					},
					slowparty = { 
						type = "toggle",
						name = L["Show Slow Messages in Party"],
						desc = L["Toggles the display of slow messages in party chat"],
						order = 13,
						get = function() return db.showSlowInParty end,
						set = function ()db.showSlowInParty = not db.showSlowInParty end,
					},
					slowraid = {
						type = "toggle",
						name = L["Show Slow Messages in Raid"],
						desc = L["Toggles the display of slow messages in raid chat"],
						order = 14,
						get = function() return db.showSlowInRaid end,
						set = function ()db.showSlowInRaid = not db.showSlowInRaid end,
					},
					slowcust = {
						type = "toggle",
						name = L["Show Slow Messages in Custom Channel"],
						desc = L["Toggles the display of slow messages in the custom channel"],
						order = 15,
						get = function() return db.showSlowInCust end,
						set = function ()db.showSlowInCust = not db.showSlowInCust end,
					},
				},
			},
			portops = {
				type = "group",
				name = L["Portal Options"],
				desc = L["Show Portal Options"],
				order = 2,
				args = {
					portmsg = { 
						type = "text",
						name = L["Portal Message"],
						desc = L["Message to be displayed when opening a portal, %s will be replace with the city name"],
						usage = L["<Your message>"],
						order = 1,
						get = function() return db.portMessage end,
						set = function (newValue)db.portMessage = newValue end,
					},
					portparty = {
						type = "toggle",
						name = L["Show Portal Messages in Party"],
						desc = L["Toggles the display of portal messages in party chat"],
						order = 2,
						get = function() return db.showPortInParty end,
						set = function ()db.showPortInParty = not db.showPortInParty end,
					},
					portraid = {
						type = "toggle",
						name = L["Show Portal Messages in Raid"],
						desc = L["Toggles the display of portal messages in raid chat"],
						order = 3,
						get = function() return db.showPortInRaid end,
						set = function ()db.showPortInRaid = not db.showPortInRaid end,
					},
					portcust = {
						type = "toggle",
						name = L["Show Portal Messages in Custom Channel"],
						desc = L["Toggles the display of portal messages in the custom channel"],
						order = 4,
						get = function() return db.showPortInCust end,
						set = function ()db.showPortInCust = not db.showPortInCust end,
					},
				},
			},	
		},
	}
	MageAnnounce:RegisterChatCommand(L["Slash-Commands"], self.options)
	------------ End Slash Commands ------------
	------------ Defaults ------------
	MageAnnounce:RegisterDB("MageAnnounceDB", "MageAnnounceDBPC")
	MageAnnounce:RegisterDefaults("profile", {
	polyMessage = L["Now Polymorphing %t"],
	polyWarning = L["I am casting Polymorph on your target. Please stop attacking!"],
	csMessage = L["Counterspelling %t"],
	slowMessage = L["Slowing %t"],
	portMessage = L["Incoming Portal to %s"],
	customChan = "",
	showInBGs = true,
	showOutside = true,
	showPolyInRaid = true, -- Polymorph
	focusMacro = false,
	showPolyInParty = true,
	showPolyInCust = true,
	sendPolyWhisper = false,
	showCSInRaid = true, -- Counterspell
	showCSInParty = true,
	showCSInCust = true,
	showSlowInRaid = true, -- Slow
	showSlowInParty = true,
	showSlowInCust = true,
	showPortInParty = true, -- Portals
	showPortInRaid = true,
	showPortInCust = true,
	} )
	------------ End Defaults ------------
	db = self.db.profile
end

function MageAnnounce:OnEnable()
	waterfall:Register("MageAnnounce", "aceOptions", self.options, "title","MageAnnounce Options")
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	MageAnnounce:PARTY_MEMBERS_CHANGED()
	MageAnnounce:RAID_ROSTER_UPDATE()
	MageAnnounce:ZONE_CHANGED_NEW_AREA()
end

function MageAnnounce:PARTY_MEMBERS_CHANGED() -- Check to see if in party
	if GetNumPartyMembers() > 0 then
		isInParty = true
	else
		isInParty = false
	end
end
function MageAnnounce:RAID_ROSTER_UPDATE() -- Check to see if in raid
	if GetNumRaidMembers() > 0 then
		isInRaid = true
	else 
		isInRaid = false
	end
end

function MageAnnounce:ZONE_CHANGED_NEW_AREA()
	inInstance, instanceType = IsInInstance()
	if inInstance and instanceType == "pvp" then
		InBG = true
	elseif not inInstance then
		InBG = false
	end
end

function MageAnnounce:UNIT_SPELLCAST_START(caster, spellName)
	if caster ~= "player" then 
		return 
	end
	if not inInstance and db.showOutside == false then
		return
	end
	if db.customChan then
		chanid, _ = GetChannelName(db.customChan)
	end
	if spellName then
		if strfind (spellName, L["Polymorph"]) then
			if db.focusMacro then
				target = "focus"
			else
				target = "target"
			end
			sheeped = CheckIfSheeped (target)
			if not sheeped then
				message =  tokenreplace ("%d", db.slowMessage,"target")
				message =  tokenreplace ("%l", message,"target")
				message =  tokenreplace ("%s", message,"target")
				message =  tokenreplace ("%r", message,"target")
				message =  tokenreplace ("%c", message,"target")

				if isInParty and not isInRaid and db.showPolyInParty then -- Polymorph Party Messages
					SendChatMessage(message , "PARTY", nil, ""); 

				end
				if isInRaid and db.showPolyInRaid and WarnNum == 1 then -- Polymorph Raid Messages
					SendChatMessage(message , "RAID", nil, "");
					WarnNum = WarnNum + 1
				elseif WarnNum >= 2 then
					WarnNum = 1
				end
				if db.showPolyInCust and (chanid > 0) then -- Custom Channel Message
					SendChatMessage(message , "CHANNEL", nil, chanid);
				end
				if IsInBG and db.showInBGs then -- Polymorph BG Messages
					SendChatMessage(message , "BATTLEGROUND", nil, ""); 

				end
				message = ""

				if db.sendPolyWhisper then -- Polymorph Warning Messages
					message =  tokenreplace ("%d", db.polyWarning,target)
					message =  tokenreplace ("%l", message,target)
					message =  tokenreplace ("%s", message,target)
					message =  tokenreplace ("%r", message,target)
					message =  tokenreplace ("%c", message,target)
					for i=0,40 do
						if not UnitIsUnit("Party"..i,"player") then
							if UnitIsUnit("Party"..i.."Target",target) then
								local n,r=UnitName("Party"..i);
								SendChatMessage(message , "WHISPER" , nil, n);
							end
						end
					end
					
				end
				
				message = ""
			end
		end
		if strfind(spellName, L["Portal"]) then
			city = gsub (spellName, L["Portal"]..": ", "")
			message = gsub (db.portMessage, "%%s", city)
			if isInParty and not isInRaid and db.showPortInRaid then
				SendChatMessage(message, "PARTY", nil, ""); 
			end
			if isInRaid and db.showPortInRaid and WarnNum == 1 then
				SendChatMessage(message, "RAID", nil, ""); 
				WarnNum = WarnNum + 1
			elseif WarnNum >= 2 then
				WarnNum = 1
			end
			if db.showPortInCust and (chanid > 0) then -- Custom Channel Message
				SendChatMessage(message , "CHANNEL", nil, chanid);
			end
			if IsInBG and db.showInBGs then
				SendChatMessage(message, "BATTLEGROUND", nil, ""); 
			end
			message = ""
		end
	end
end
function MageAnnounce:UNIT_SPELLCAST_SUCCEEDED(caster, spellName)
	if caster ~= "player" then 
		return 
	end
	if not inInstance and db.showOutside == false then
		return
	end
	if strfind (spellName, L["Counterspell"]) then
		message =  tokenreplace ("%d", db.slowMessage,"target")
		message =  tokenreplace ("%l", message,"target")
		message =  tokenreplace ("%s", message,"target")
		message =  tokenreplace ("%r", message,"target")
		message =  tokenreplace ("%c", message,"target")

		if isInParty and not isInRaid and db.showCSInParty then -- Counterspell Party Messages
			SendChatMessage(message, "PARTY", nil, ""); 
		end
		if isInRaid and db.showCSInRaid then -- Counterspell Raid Messages
			SendChatMessage(message, "RAID", nil, "");
		end
		if db.showCSInCust and (chanid > 0) then -- Custom Channel Message
			SendChatMessage(message, "CHANNEL", nil, chanid);
		end
		if IsInBG and db.showInBGs then -- Slow BG Messages
			SendChatMessage(message, "BATTLEGROUND", nil, "");
		end
		message = ""
	end
	if spellName == L["Slow"] then
		message =  tokenreplace ("%d", db.slowMessage,"target")
		message =  tokenreplace ("%l", message,"target")
		message =  tokenreplace ("%s", message,"target")
		message =  tokenreplace ("%r", message,"target")
		message =  tokenreplace ("%c", message,"target")

		if isInParty and not isInRaid and db.showSlowInParty then -- Slow Party Messages
			SendChatMessage(message, "PARTY", nil, ""); 

		end
		if isInRaid and db.showSlowInRaid then -- Slow Raid Messages
			SendChatMessage(message, "RAID", nil, "");
		end
		if db.showSlowInCust and (chanid > 0) then -- Custom Channel Message
			SendChatMessage(message, "CHANNEL", nil, chanid);
		end
		if IsInBG and db.showInBGs then -- Slow BG Messages
			SendChatMessage(message, "BATTLEGROUND", nil, "");
		end
		message = ""
	end
end
------------ End Main Code ------------

------------ Functions ------------
function CheckIfSheeped(unit)
	id = 1
	while id <= 40 do
		local name, _, texture = UnitDebuff(unit, id);
		if not name then
			break;
		end
		if string.match(texture, "Spell_Nature_Polymorph") then
			return true;
		end
		id = id + 1
	end
end

function MageAnnounce:SetCustomChan(newValue) -- Set Custom Channel
	local id, _ = GetChannelName(newValue)
	if newValue and id == 0 then
		JoinChannelByName(newValue)
		ChatFrame_AddChannel(DEFAULT_CHAT_FRAME, newValue)
	end
	db.customChan = newValue
end

function tokenreplace(token, message, target)
	if token == "%d" then
		local genderTable = { "Unknown", "male", "female" };
		local sex = genderTable[UnitSex(target)]
		local level = UnitLevel(target)
		local tokenrace,tokenraceEn = UnitRace(target)
		local tokenclass,tokenclassEn = UnitClass(target)
		detailstr = level .. "-";
		if UnitIsPlayer(target) then
			detailstr = detailstr .. tokenrace .. "-" .. tokenclass .. "-";
		end
		detailstr = detailstr .. sex;
		newmessage = gsub (message, "%%d", detailstr)
		return newmessage;
	end
	if token == "%s" then
		local genderTable = { "Unknown", "male", "female" };
		local sex = genderTable[UnitSex(target)]
		newmessage = gsub (message, "%%s", sex)
		return newmessage;
	end
	if token == "%l" then
		local level = UnitLevel(target)
		newmessage = gsub (message, "%%l", level)
		return newmessage;
	end
	if token == "%r" then
		local race,raceEN
		if UnitIsPlayer(target) then
			race,raceEn = UnitRace(target);
			else race = "n/a";
		end
		newmessage = gsub (message, "%%r", race)
		return newmessage;
	end
	if token == "%c" then
		local class,classEn
		if UnitIsPlayer(target) then
			class,classEn = UnitClass(target);
			else class = "n/a";
		end
		newmessage = gsub (message, "%%c", class)
		return newmessage;
	end
end
------------ End Functions ------------